import Vue from 'vue'
import VueRouter from 'vue-router'
import store from '@/store'
import { generateIndexRouter } from './generateIndexRouter'
import UserLayout from '@/components/layouts/UserLayout'

import NProgress from 'nprogress'
import 'nprogress/nprogress.css'

NProgress.configure({ showSpinner: false })

// hack router push callback
const originalPush = VueRouter.prototype.push
VueRouter.prototype.push = function push(location, onResolve, onReject) {
  if (onResolve || onReject) return originalPush.call(this, location, onResolve, onReject)
  return originalPush.call(this, location).catch(err => err)
}

Vue.use(VueRouter)

const whiteList = ['/user/login', '/user/register', '/user/register-result', '/user/alteration']
const constantRoutes = [
  {
    path: '/user',
    component: UserLayout,
    redirect: '/user/login',
    hidden: true,
    children: [
      {
        path: '/user/login',
        name: 'login',
        component: () => import(/* webpackChunkName: "user" */ '@/views/user/Login'),
      },
    ],
  },
  {
    path: '/404',
    component: () => import(/* webpackChunkName: "404" */ '@/views/exception/404'),
  },
]

const router = new VueRouter({
  routes: constantRoutes,
  // scrollBehavior(to, from, savedPosition) {
  //   if (savedPosition && to.meta.keepAlive) {
  //     return savedPosition
  //   } else {
  //     return {
  //       x: 0,
  //       y: 0,
  //     }
  //   }
  // },
})

let asyncRoutes = []
router.beforeEach((to, from, next) => {
  NProgress.start()
  /**
   * 1.有token
   *    |-- 1.1 登录页 ==> 跳转首页
   *    |-- 1.2 非登录页 ==> 判断是否有权限菜单
   *             |-- 1.2.1 没有权限菜单 ==> 获取权限, 生成动态路由, 获取用户信息, 放行
   *             |-- 1.2.2 有权限菜单  ==> 放行
   * 2. 没有token
   *    |-- 2.1 在白名单  ==> 放行
   *    |-- 2.2 不在白名单 ==> 跳转登录页
   */
  if (store.state.user.token || Vue.ls.get('token')) {
    console.log('打印', Vue.ls.get('token'))
    /* has token */
    hasToken(to, from, next)
  } else {
    if (whiteList.indexOf(to.path) !== -1) {
      next()
    } else {
      next({ path: '/user/login' })
    }
  }
})

function hasToken(to, from, next) {
  if (to.path === '/user/login') {
    next({ path: '/home' })
  } else {
    if (!store.state.user.menu[0]) {
      generateAsyncRouters(to, from, next)
    } else {
      next()
    }
  }
}

function generateAsyncRouters(to, from, next) {
  store
    .dispatch('user/getPermissionList')
    .then(menu => {
      asyncRoutes = generateIndexRouter(menu)
      router.addRoutes(asyncRoutes)
      store.dispatch('user/getUserInfo')
      next({ path: to.path })
    })
    .catch(err => {
      console.log(err)
      Vue.prototype.$error({
        title: '请求用户信息失败',
        destroyOnClose: true,
        onOk: function() {
          if (from.path === '/user/login') {
            store.commit('user/setToken', '')
            window.location.reload()
          } else {
            store.dispatch('user/logout')
          }
        },
      })
    })
}

router.afterEach(() => {
  NProgress.done()
})

export default router
