import Vue from 'vue'
import { getUserInfo, getUserPermission } from '@/api/user'

// 递归添加每级路由到mapMenu根目录, 并添加父路由
function handleMenu(mockMenu = [], mapMenu = {}, parent) {
  for (const item of mockMenu) {
    item.parent = parent
    mapMenu[item.path] = item
    if (item.children) {
      handleMenu(item.children, mapMenu, item)
    }
  }
  return mapMenu
}

let user = {
  state: {
    isAdmin: false,
    username: '',
    avatar: '',
    info: {},
    token: '',
    menu: [], // 菜单
    buttonAuth: [], // 权限: 新增/删除/编辑, 按钮
  },

  getters: {
    mapMenu: state => {
      if (!state.menu[0]) {
        return []
      }
      return handleMenu(state.menu)
    },
  },

  mutations: {
    setToken(state, value) {
      state.token = value
      Vue.ls.set('token', value)
    },
    // 保存用户菜单
    saveMenuList(state, value) {
      state.menu = value
    },
    saveButtonAuth(state, value) {
      let actionList = value.map(i => i.action)
      state.buttonAuth = actionList
      // 把按钮/权限存在section, 退出登录时记得清空
      sessionStorage.setItem('buttonAuth', JSON.stringify(actionList))
    },
    saveIsAdmin(state, value) {
      state.isAdmin = value
      if (value) {
        state.buttonAuth.unshift('admin')
      }
    },
    saveUserName(state, value) {
      state.username = value
    },
    saveAvatar(state, value) {
      state.avatar = value
    },
    saveInfo(state, value) {
      state.info = value
    },
  },

  actions: {
    logout({ commit }) {
      return new Promise(resolve => {
        commit('setToken', '')
        commit('saveMenuList', [])
        window.sessionStorage.clear()
        window.location.reload()
        resolve()
      })
    },

    // 获取用户权限
    async getPermissionList({ commit }) {
      const { errcode, data } = await getUserPermission()
      if (errcode === 0 && data) {
        const { menu, buttonAuth } = data
        // 如果二菜单全部是隐藏的, 那么一级菜单也隐藏
        if (menu && menu.length > 0) {
          menu.forEach(item => {
            if (item['children']) {
              let hasChildrenMenu = item['children'].filter(i => {
                return !i.hidden || i.hidden == false
              })
              if (hasChildrenMenu == null || hasChildrenMenu.length == 0) {
                item['hidden'] = true
              }
            }
          })
        }
        commit('saveMenuList', menu)
        commit('saveButtonAuth', buttonAuth)
        return new Promise(resolve => {
          resolve(menu)
        })
      }
    },

    async getUserInfo({ commit }) {
      const { errcode, data } = await getUserInfo()
      if (errcode === 0 && data) {
        let { isAdmin, username, avatar, info } = data
        commit('saveIsAdmin', isAdmin)
        commit('saveUserName', username)
        commit('saveAvatar', avatar)
        commit('saveInfo', info)
        return new Promise(resolve => resolve())
      }
    },
  },
}

export default user
