import axios from 'axios'
import store from '@/store'
import notification from 'ant-design-vue/es/notification'

const axiosInstance = axios.create({
  baseURL: process.env.VUE_APP_API_BASE_URL,
  timeout: 6000,
})

const errorHandler = err => {
  notification.error({
    message: '请求错误',
    description: err.message,
  })
  return { err }
}

axiosInstance.interceptors.request.use(
  config => {
    // 头部增加 token
    if (store.state.user.token) {
      config.headers.Authorization = 'Bearer ' + store.state.user.token
    }

    return config
  },
  err => {
    return Promise.reject(err)
  }
)

axiosInstance.interceptors.response.use(response => {
  // 响应没有data属性，说明网络请求失败
  if (!response.data) {
    return response
  }
  if (response.data.errcode) {
    // 如果response.data.errcode有值并且不为0的时候返回一个包含错误信息的对象
    return errorHandler({ message: response.data.errmsg })
  }
  return response.data
}, errorHandler)

export default axiosInstance
